package view;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javafx.beans.binding.Bindings;
import javafx.beans.property.StringProperty;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;
import javafx.util.converter.NumberStringConverter;
import model.*;

import javax.swing.text.LabelView;

public class TreeViewCtrl extends FXMLView {
    @FXML
    private TreeView<CapteurAbstrait> treeView;

    @FXML
    private Label idLabel;
    @FXML
    private Label nameLabel;
    @FXML
    private TextField nameTextField;
    @FXML
    private Label temperatureLabel;

    @FXML
    private ComboBox strategyOptions;

    private ArrayList<CapteurAbstrait> capteurAbstraits;


    public TreeViewCtrl(ArrayList<CapteurAbstrait> capteurs, String url, String title) throws IOException {
        super(url,title);
        this.capteurAbstraits = capteurs;

        TreeItem<CapteurAbstrait> root = new TreeItem<>();
        treeView.setShowRoot(false);
        root.setExpanded(true);
        //treeView.setRoot(root);
        TreeItemFactoryVisitor treeItemFactoryVisitor = new TreeItemFactoryVisitor();

        treeView.setVisible(true);

        if (root != null && treeView != null) {
            treeView.setRoot(root);
            for (CapteurAbstrait capteur : capteurAbstraits) {
                TreeItem<CapteurAbstrait> item = capteur.accept(treeItemFactoryVisitor);
                root.getChildren().add(item);
            }
        }
//        for(CapteurAbstrait capteur: capteurAbstraits){
//            TreeItem<CapteurAbstrait> item = capteur.accept(treeItemFactoryVisitor);
//            root.getChildren().add(item);
//        }
        this.initializeCapteur();

    }

    public void initializeCapteur() {
        strategyOptions.getItems().addAll("CPU", "Random");

        treeView.setCellFactory(param -> new TreeCell<CapteurAbstrait>() {
            @Override
            protected void updateItem(CapteurAbstrait capteur, boolean empty) {
                super.updateItem(capteur, empty);
                if (empty || capteur == null) {
                    textProperty().unbind();
                    setText("");
                } else {
                    setText(capteur.getNom().getValue());
                    String image = "/images/random.png";
                    if (capteur.getClass() == CapteurVirtuel.class) {
                        image = "/images/virtual.png";
                    } else if (capteur.getClass() == UnitCapteur.class && ((UnitCapteur) capteur).getStrategie().getClass() == StrategyCPU.class) {
                        image = "/images/cpu.png";
                    } else if (capteur.getClass() == UnitCapteur.class && ((UnitCapteur) capteur).getStrategie().getClass() == StrategyRandom.class) {
                        image = "/images/random.png";
                    }

                    setGraphic(new ImageView(new Image(image)) {{
                        setFitHeight(40);
                        setFitWidth(40);
                    }});
                }
            }
        });
        strategyOptions.setVisible(false);
        treeView.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            if (oldValue != null) {
                CapteurAbstrait capteur = oldValue.getValue();
                idLabel.textProperty().unbindBidirectional(capteur.getId());
                nameLabel.textProperty().unbindBidirectional(capteur.getNom());
                temperatureLabel.textProperty().unbind();
                if(capteur instanceof UnitCapteur){
                    strategyOptions.setVisible(false);
                }
            }
            if (newValue != null) {
                CapteurAbstrait capteur = newValue.getValue();
                if (capteur instanceof UnitCapteur) {
                    strategyOptions.setVisible(true);
                } else {
                    strategyOptions.setVisible(false);
                }
                idLabel.textProperty().bindBidirectional(capteur.getId());
                nameLabel.textProperty().bindBidirectional(capteur.getNom());
                temperatureLabel.textProperty().bind(Bindings.format("%.2f",capteur.getTemp()));
                nameTextField.setOnAction(event -> {
                    capteur.setNom(nameTextField.getText());
                });
                nameTextField.setText(capteur.getNom().getValue());
                capteur.getNom().addListener((obs, oldName, newName) -> {
                    capteur.setNom(newName);
                    newValue.setValue(capteur);
                });
                if(capteur instanceof UnitCapteur){
                    strategyOptions.setVisible(true);
                }
            }
        });
    }

    public TreeView<CapteurAbstrait> getTreeView() {
        return treeView;
    }
}
